#include "OOTCPApi.h"

#define LOCIP "150.140.186.93"	// default local IP
#define REMIP "150.140.186.106"	// default remote IP

#define MAX_WORDS 52429
#define MAX_WORD_LEN 100

pthread_t app_thread;
void * app_thread_code(void * t);
int sendall(Connection * con, char * buf, int len);
int recvall(Connection * con, char * buf, int len);

char B[MAX_WORDS][MAX_WORD_LEN];

int init_module(void) {

	pthread_create(&app_thread, NULL, app_thread_code, 0);
	
	return 0;
}

int cleanup_module(void) {

	pthread_delete_np(app_thread);
	rtl_printf("Asta la vista.\n");

	return 0;
} 

void * app_thread_code(void * t) {

	int i = 0;

	struct sched_param p;
	p.sched_priority = 4;
	pthread_setschedparam(pthread_self(), SCHED_RR, &p);

	printf("Application thread started\n");
	Connection * con;
	if((con = ConnectionHandler::allocate(pthread_self())) == NULL){
		printf("Error in allocating con\n");
		return (void *) NULL;
	}

	if(con->activeOpen(LOCIP, 44, REMIP, 3210) == OK)
		printf("Connection established!\n");
	else{
		printf("Error in activeOpen().\n");
		ConnectionHandler::deallocate(con);
		return (void *) NULL;
	}
	
	for(i = 0 ; i < MAX_WORDS ; i++)
		recvall(con, B[i], MAX_WORD_LEN);
	
	printf("App: Received everything!\n");

	for(i = 0 ; i < MAX_WORDS ; i++)
		sendall(con, B[i], MAX_WORD_LEN);
		
	printf("App: Sent everything!\n");

	ConnectionHandler::printStatus();

	while(con->receive(" ", 1) != 0)	// wait for server to begin closing
		usleep(100000);

	if(con->close() == OK)
		printf("Connection closed.\n");
	else
		printf("Error in close().\n");
	
	usleep(10000);
	ConnectionHandler::printStatus();

	return (void *) NULL;
}


int sendall(Connection * con, char * buf, int len){

	int total = 0; // how many bytes we've sent
	int bytesleft = len; // how many we have left to send
	int n;

	while(total < len) {
		n = con->send(buf+total, bytesleft);
		if (n == -1) { break; }
		total += n;
		bytesleft -= n;
	}

	return n == -1 ? -1 : 0; // return -1 on failure, 0 on success
}


int recvall(Connection * con, char * buf, int len){

	int total = 0; // how many bytes we've received
	int bytesleft = len; // how many we have left to receive
	int n;

	while(total < len) {
		n = con->receive(buf+total, bytesleft);
		if (n == -1) { break; }
		total += n;
		bytesleft -= n;
	}
	

	return n == -1 ? -1 : 0; // return -1 on failure, 0 on success
}

