#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <time.h>
#include <winsock.h>


#define MYPORT 3210 // the port users will be connecting to
#define BACKLOG 1 // how many pending connections queue will hold

#define MAX_WORDS 52429
#define MAX_WORD_LEN 100

int sendall(int sockfd, char * buf, int len);
int recvall(int sockfd, char * buf, int len);

char A[MAX_WORDS][MAX_WORD_LEN];
char B[MAX_WORDS][MAX_WORD_LEN];


void main(void){

	int sockfd, new_fd; // listen on sock_fd, new connection on new_fd
	struct sockaddr_in my_addr; // my address information
	struct sockaddr_in their_addr; // connectors address information
	int sin_size;

	WSADATA wsaData; // if this doesnt work
	//WSAData wsaData; // then try this instead
	if (WSAStartup(MAKEWORD(1, 1), &wsaData) != 0) {
		fprintf(stderr, "WSAStartup failed.\n");
		exit(1);
	}

	if ((sockfd = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
		perror("socket");
		exit(1);
	}

	my_addr.sin_family = AF_INET; // host byte order
	my_addr.sin_port = htons(MYPORT); // short, network byte order
	my_addr.sin_addr.s_addr = INADDR_ANY; // automatically fill with my IP
	memset(&(my_addr.sin_zero), '\0', 8); // zero the rest of the struct


	if (bind(sockfd, (struct sockaddr *)&my_addr, sizeof(struct sockaddr)) == -1) {
		perror("bind");
		exit(1);
	}

	if (listen(sockfd, BACKLOG) == -1) {
		perror("listen");
		exit(1);
	}
	
	sin_size = sizeof(struct sockaddr_in);

	clock_t t1, t2, t3;
	FILE *words_file, *result_file;
	int i = 0;

    words_file = fopen("words.txt", "r");
	if(words_file == NULL){
		printf("Cannot open words.txt\n");
		exit(1);
	}

	for(i = 0 ; i < MAX_WORDS ; i++){
		if(fgets(A[i], MAX_WORD_LEN, words_file)==NULL)
			perror("fgets");
	}
	fclose(words_file);

	printf("Waiting on port %d...\n", MYPORT);
	if ((new_fd = accept(sockfd, (struct sockaddr *)&their_addr, &sin_size)) == -1) {
		perror("accept");
		exit(1);
	}
	printf("Got connection from %s:%d\n", inet_ntoa(their_addr.sin_addr), ntohs(their_addr.sin_port));


	t1 = clock();

	for(i = 0 ; i < MAX_WORDS ; i++){
		if (sendall(new_fd, A[i], MAX_WORD_LEN) == -1)
			perror("send");
	}
	t2 = clock();

	for(i = 0 ; i < MAX_WORDS ; i++){
		if (recvall(new_fd, B[i], MAX_WORD_LEN) == -1)
			perror("recv");
	}

	t3 = clock();

	printf("Elapsed time for sending: %g seconds.\n", (long double)(t2-t1)/CLOCKS_PER_SEC);
	printf("Elapsed time for receiving: %g seconds.\n", (long double)(t3-t2)/CLOCKS_PER_SEC);	
	printf("Total elapsed time: %g seconds.\n", (long double)(t3-t1)/CLOCKS_PER_SEC);	
	
	
	result_file=fopen("result.txt", "w");
	if(result_file == NULL){
		printf("Cannot create result.txt\n");
		exit(1);
	}
	for(i = 0 ; i < MAX_WORDS ; i++)
		fprintf(result_file, "%s", B[i]);
	fclose(result_file);


	printf("Press enter to close connection\n");
	fflush(stdin);
	getchar();
	closesocket(new_fd);
	closesocket(sockfd);

}

int sendall(int sockfd, char * buf, int len){

	int total = 0; // how many bytes we've sent
	int bytesleft = len; // how many we have left to send
	int n;

	while(total < len) {
		n = send(sockfd, buf+total, bytesleft, 0);
		if (n == -1) { break; }
		total += n;
		bytesleft -= n;
	}

	return n == -1 ? -1 : 0; // return -1 on failure, 0 on success
}


int recvall(int sockfd, char * buf, int len){

	int total = 0; // how many bytes we've received
	int bytesleft = len; // how many we have left to receive
	int n;

	while(total < len) {
		n = recv(sockfd, buf+total, bytesleft, 0);
		if (n == -1) { break; }
		total += n;
		bytesleft -= n;
	}
	

	return n == -1 ? -1 : 0; // return -1 on failure, 0 on success
}
